function parseSettingsFromJson(name) {
    let $block = $('#' + name);

    let settings = {};

    let settingsRaw = $.parseJSON($block.html() || '{}');
    $block.remove();
    return settingsRaw;
}

$(() => {
    var attendance = parseSettingsFromJson("json_attendance");
    console.log(attendance);
    var students = parseSettingsFromJson("json_students");
    console.log(students);
    var groups = parseSettingsFromJson("json_groups");
    console.log(groups);
    var discipline = parseSettingsFromJson("json_discipline");
    console.log(discipline);
    var lessons = parseSettingsFromJson("json_lessons");
    console.log(lessons);
    var lessonTypes = parseSettingsFromJson("json_lessonTypes");
    console.log(lessonTypes);
    var columnDescription = [
        {
            title: "ФИО",
            field: "fullName",
            frozen: true,
            width: 160
        },

    ];

    var studentList = Array();

    var fillFullNames = function (studentList) {
        studentList.forEach((student) => {
            student.fullName = [student.LastName, student.FirstName, student.SecondName].join(' ');
        })
    }

    let table = null;

    Object.keys(lessons).forEach((num) => {
        lesson = lessons[num];
        lessonDate = new Date(lesson.LessonDate);
        lessonFormattedDate = lessonDate.getDate().toString() + '-' + (lessonDate.getMonth() + 1).toString();

        lessonDescription = {
            //title: lesson.LessonDate,
            title: `<input type="text" id="lesson_input_${lesson.ID}" class="lesson_input" value="${lessonFormattedDate}") }}">
                    <div>
                        <input type="text" id="lesson_name_${lesson.ID}" class="lesson_name_input" value="${lesson.LessonName}">
                    </div>
                    <div class="delete_lesson" id="lesson_delete_${lesson.ID}"><span class="delete_lesson_icon"></span></div>`,
            field: "Lesson" + lesson.ID,
            headerVertical: false,
            // editor: "tickCross",
            formatter: "tickCross",
            sortable: false, headerSort: false
        };
        columnDescription.push(lessonDescription);
    });

    console.log(columnDescription);

    // columnDescription.push({
    //     title: "Итого",
    //     field: "total",
    //     headerVertical: false,
    // });

    Object.keys(students).forEach((groupID) => {
        fillFullNames(students[groupID]);
        students[groupID].forEach((student) => {
            student.groupID = groupID;
        });
        studentList.push(...students[groupID]);
    });

    studentList.forEach((student) => {
        recordBookID = student.RecordBookID;
        let totalAttendance = 0;
        if (attendance.hasOwnProperty(recordBookID)) {
            Object.keys(lessons).forEach((num) => {
                lesson = lessons[num];
                if (attendance[recordBookID].hasOwnProperty(lesson.ID)) {
                    let propName = "Lesson" + lesson.ID;
                    let attended = attendance[recordBookID][lesson.ID];
                    // = (bool) attended;
                    student[propName] = !!attended;
                    totalAttendance += attended;
                }
            });
        }
        student.total = totalAttendance;
    });

    const editLessonDate = function (lessonID, dateText, onSuccess, onFail) {
        let currentYear = discipline.semesterYear;
        if (discipline.semesterNum === 2) {
            currentYear += 1;
        }
        let data = {};
        data.lessonID = lessonID;
        formattedDate = dateText.split('-').reverse().join('-');
        data.date = currentYear + "_" + formattedDate;

        $.postJSON(g_URLdir + "handler/journal/SetLessonDate", data).success(() => {
            Popup.success("Дата успешно установлена");
            onSuccess();
        }).fail(jqXHR => {
            onFail();
            let status = parseInt(jqXHR.status);
            try {
                if (status != 400) throw null;
                let message = JSON.parse(jqXHR.responseText).message;
                if (!message) throw null;
                Popup.error(message);
            } catch (error) {
                Popup.error("Не удалось сменить дату занятия");
            }
        });
    };

    const createLesson = function (disciplineID, dateText, lessonType, onSuccess, onFail) {
        let data = {};
        data.disciplineID = disciplineID;
        data.date = dateText;
        data.lessonType = lessonType;

        $.postJSON(g_URLdir + "handler/journal/CreateLesson", data).success((response) => {
            Popup.success("Занятие успешно создано");
            onSuccess(response["lessonID"]);
        }).fail(jqXHR => {
            onFail();
            let status = parseInt(jqXHR.status);
            try {
                if (status != 400) throw null;
                let message = JSON.parse(jqXHR.responseText).message;
                if (!message) throw null;
                Popup.error(message);
            } catch (error) {
                Popup.error("Не удалось создать занятие");
            }
        });
    };

    const deleteLesson = function (lessonID, onSuccess, onFail) {
        let data = {};
        data.lessonID = lessonID;

        $.postJSON(g_URLdir + "handler/journal/DeleteLesson", data).success((response) => {
            Popup.success("Занятие успешно удалено");
            onSuccess(response["lessonID"]);
        }).fail(jqXHR => {
            onFail();
            let status = parseInt(jqXHR.status);
            try {
                if (status != 400) throw null;
                let message = JSON.parse(jqXHR.responseText).message;
                if (!message) throw null;
                Popup.error(message);
            } catch (error) {
                Popup.error("Не удалось удалить занятие");
            }
        });
    };

    const onDateInput = function ($elem, dateText, oldDateText) {
        //console.log($elem);
        //console.log(dateText);
        //console.log(oldDateText);
        if (dateText !== oldDateText) {
            let inputId = $elem[0].getAttribute("id");
            let lessonID = parseInt(inputId.substring(13, 25));
            editLessonDate(lessonID, dateText, function () {
            }, function () {
                $elem.val(oldDateText);
            });
        }
    };

    let sendAttendance = function (recordBookID, lessonID, url, data, reset, successText, errorText) {
        data.recordBookID = recordBookID;
        data.disciplineID = discipline.ID;
        data.lessonID = lessonID;

        $.postJSON(g_URLdir + url, data).success(() => {
            // this.recountScores($cell);
            Popup.success(successText);
        }).fail(jqXHR => {
            let status = parseInt(jqXHR.status);
            try {
                if (status != 400) throw null;
                let message = JSON.parse(jqXHR.responseText).message;
                if (!message) throw null;
                Popup.error(message);
            } catch (error) {
                Popup.error(errorText);
            }
        });
    };

    let setAttendance = function (recordBookID, lessonID, reset, oldValue, newValue) {
        let successText = 'Присутствие установлено';
        let errorText = 'Не удалось установить присутствие';
        let attendance = 1;
        if (!newValue) {
            successText = 'Отсутствие установлено';
            errorText = 'Не удалось установить отсутствие';
            attendance = 0;
        }
        sendAttendance(recordBookID, lessonID, 'handler/journal/SetAttendance', {attendance},
            reset, successText, errorText);
    };

    let editAttendance = function (cell) {
        //console.log(cell);
        let student = cell._cell.row.data;
        // TODO: no magic numbers, get numbers after "Lesson"
        let lessonID = parseInt(cell._cell.column.field.substring(6, 20));
        let oldValue = cell._cell.oldValue;
        let newValue = cell._cell.value;
        setAttendance(student.RecordBookID, lessonID, () => {
            cell.setValue(oldValue);
        }, oldValue, newValue);
    };

    let undoAttendance = function(action, component, data) {
        console.log(action);
        console.log(component);
        console.log(data);
        if (action === "cellEdit") {
            let oldValue = data.oldValue;
            let newValue = data.newValue;
            let student = component._cell.row.data;
            let lessonID = parseInt(component._cell.column.field.substring(6, 20));
            setAttendance(student.RecordBookID, lessonID, () => {
                component._cell.setValue(newValue);
            }, newValue, oldValue);
        }
    };

    let redoAttendance = function(action, component, data) {
        console.log(action);
        console.log(component);
        console.log(data);
        if (action === "cellEdit") {
            let oldValue = data.oldValue;
            let newValue = data.newValue;
            let student = component._cell.row.data;
            let lessonID = parseInt(component._cell.column.field.substring(6, 20));
            setAttendance(student.RecordBookID, lessonID, () => {
                component._cell.setValue(oldValue);
            }, oldValue, newValue);
        }
    };

    let onCellClick = function(e, cell) {
        oldValue = cell.getValue();
        if (oldValue) {
            cell.setValue(false)
        } else {
            cell.setValue(true)
        }
    };

    table = new Tabulator("#studentsAttendance", {
        columnVertAlign: "bottom", //align header contents to bottom of cell
        data: studentList,
        index: "RecordBookID",
        // TODO: height of screen so scroll bar shows by default
        height: "80%",
        //layout: "fitColumns",
        persistentLayout: true,
        persistentSort: true,
        persistenceID: "DisciplineJournal" + discipline.ID,
        clipboard: true,
        history: true,
        cellClick: onCellClick,
        cellEdited: editAttendance,
        historyUndo: undoAttendance,
        historyRedo: redoAttendance,
        clipboardPasteAction: "replace",
        columns: columnDescription,
        groupBy: "GroupID",
        groupHeader: function (value, count, data, group) {
            groupInfo = groups[value];
            return groupInfo.GroupNum + " группа " + groupInfo.GradeID + " курса";
        }
    });

    let currentYear = discipline.semesterYear;
    if (discipline.semesterNum === 2) {
        currentYear += 1
    }

    const defaultDate = new Date(Date.now());
    let numericYear = parseInt(currentYear);
    if (discipline.semesterNum === "2") {
        numericYear++;
    }
    defaultDate.setFullYear(numericYear);

    // let $delete_lesson = $(".delete_lesson");
    // let $switcher = $(".delete_button_switcher");
    // $delete_lesson.hide();
    // $(".rateHeader").on("click", ".delete_button_switcher", function () {
    //     if ($(".delete_button_switcher").hasClass("delete_mode")) {
    //         $(".delete_lesson").hide();
    //         $(".delete_button_switcher").removeClass("delete_mode")
    //     } else {
    //         $(".delete_button_switcher").addClass("delete_mode");
    //         $(".delete_lesson").show();
    //     }
    // });

    $.datepicker.setDefaults(
        $.extend(
            {'dateFormat': 'dd-mm'},
            $.datepicker.regional['ru']
        )
    );

    const setLessonNameHandlers = function ($elem) {
        let self = this;
        const editLessonName = function ($cell, lessonID, newName, onSuccess, onFail) {
            $cell.turnOff();

            let data = {};
            data.lessonID = lessonID;
            data.name = newName;

            $.postJSON(g_URLdir + "handler/journal/SetLessonName", data).success(() => {
                Popup.success("Название успешно установлено");
                onSuccess();
            }).fail(jqXHR => {
                onFail();
                let status = parseInt(jqXHR.status);
                try {
                    if (status != 400) throw null;
                    let message = JSON.parse(jqXHR.responseText).message;
                    if (!message) throw null;
                    Popup.error(message);
                } catch (error) {
                    Popup.error("Не удалось сменить название занятия");
                }
            }).always(() => $cell.turnOn());
        };

        let setLessonName = function(lessonID, oldName, newName) {
            if (newName !== oldName) {
                editLessonName($elem, lessonID, newName, function() {
                }, function() {
                    $elem.val(oldName);
                });
            }
        };

        let oldNameText = $elem.val();
        $elem.on("change", function() {
            let lessonID = $(this).attr('id').substring(12,20);
            let newNameText = $(this).val();
            setLessonName(lessonID, oldNameText, newNameText);
        });
    };
    setLessonNameHandlers($(".lesson_name_input"));

    let $cell = undefined;
    const setDateTimePickerHandlers = function ($elem) {
        let oldDateText = $elem.val();
        $cell = $elem;
        //$elem.datetimepicker({
        $elem.datepicker({
            dateFormat: 'dd-mm',
            changeMonth: true,
            changeYear: false,
            defaultDate: defaultDate,
            //timeFormat: 'HH:mm:ss',
            beforeShow: function () {
                oldDateText = $(this).val();
                $cell = this;
            },
            onClose: function (dateText) {
                onDateInput($(this), dateText, oldDateText);
            }
        });
        // $elem.on("change", {}, $elem.dattetimepicker()) ломает datetimepicker в Chrome:
        // jquery-1.11.1.min.js:3 Uncaught TypeError: ((m.event.special[e.origType] || {}).handle || e.handler).apply is not a function
        //     at HTMLInputElement.dispatch (jquery-1.11.1.min.js:3)
        //     at HTMLInputElement.r.handle (jquery-1.11.1.min.js:3)
        //     at Object.trigger (jquery-1.11.1.min.js:3)
        //     at HTMLInputElement.<anonymous> (jquery-1.11.1.min.js:3)
        //     at Function.each (jquery-1.11.1.min.js:2)
        //     at m.fn.init.each (jquery-1.11.1.min.js:2)
        //     at m.fn.init.trigger (jquery-1.11.1.min.js:3)
        //     at Timepicker._updateDateTime (jquery-timepicker-addon-1.6.3.js:987)
        //     at Datepicker.$.datepicker._formatDate (jquery-timepicker-addon-1.6.3.js:1808)
        //     at Datepicker._selectDay (jquery-ui.js:8191)
        // $elem.on("change", {}, $elem.datetimepicker());
        //$elem.change().datetimepicker();
        $elem.change().datepicker();
    };
    setDateTimePickerHandlers($(".lesson_input"));

    $(".new_lesson_input").val(defaultDate.getDate().toString() + '-' + (defaultDate.getMonth()+1).toString());

    const onNewLessonDate = function (newLessonDate) {
        let newLessonType = parseInt($('#lessonSelector').val());
        if (newLessonType === 0) {
            newLessonType = 1;
        }
        formattedLessonDate = newLessonDate.split('-').reverse().join('-');

        const newDate = currentYear + "-" + formattedLessonDate;

        createLesson(discipline.ID, newDate, newLessonType, function (lessonID) {
            table.addColumn({
                title: `<input type="text" id="lesson_input_${lessonID}" class="lesson_input" value="${newLessonDate}") }}">
                        <div>
                            <input type="text" id="lesson_name_${lessonID}" class="lesson_name_input" value="">
                        </div>
                    <div class="delete_lesson" id="lesson_delete_${lessonID}"><span class="delete_lesson_icon"></span></div>`,
                field: "Lesson" + lessonID,
                headerVertical: false,
                //editor: "tickCross",
                formatter: "tickCross",
                sortable: false, headerSort: false
            }, false);
            var col = table.getColumn("Lesson"+lessonID);
            setLessonNameHandlers($(col.getElement()).find(".lesson_name_input"));
            setDateTimePickerHandlers($(col.getElement()).find(".lesson_input"));
            studentList.forEach((student) => {
                student["Lesson" + lessonID] = false;
                //table.updateData(student);
            });
            lessons.push({
                ID: lessonID, recordBookID,
                LessonDate: formattedLessonDate,
                LessonType: newLessonType
            });
        }, function () {

        });
    };

    const setNewLessonInput = function ($elem) {
        let oldDateText = $elem.val();
        $cell = $elem;
        //$elem.datetimepicker({
        $elem.datepicker({
            dateFormat: 'dd-mm',
            changeMonth: true,
            changeYear: false,
            defaultDate: defaultDate,
            //timeFormat: 'HH:mm:ss',
            beforeShow: function () {
                oldDateText = $(this).val();
                $cell = this;
            },
            onClose: function (dateText) {
                //onNewLessonDate(dateText);
            }
        });
        $elem.change().datepicker();
    };
    setNewLessonInput($(".new_lesson_input"));
    $('#new_lesson_button').on("click", function() {
        onNewLessonDate($(".new_lesson_input").val());
    });

    const onLessonDelete = function ($cell) {
        if (!confirm('Вы уверены, что хотите удалить это занятие?')) {
            return;
        }
        // TODO: no magic numbers
        const lessonID = $cell[0].getAttribute("id").substring(14,25);

        table.deleteColumn("Lesson"+lessonID);

        deleteLesson(lessonID, function (newLessonID) {
        }, function () {
        });
    };

    $('#studentsAttendance').on('click', '.delete_lesson', function () {
        onLessonDelete($(this));
    });

    class LessonFilter {
        constructor(table, $selector) {
            this.table = table;

            let lessonFilter = JSON.parse(localStorage.lessonFilter || "{}");
            let lesson_type_id = lessonFilter[discipline.ID] || 0;

            $selector.children('[value=' + lesson_type_id + ']').prop('selected', true);
            this.filterLessons(lesson_type_id);

            this.listen($selector);
        }

        filterLessons(lesson_type_id) {
            lessons.forEach((lesson)=>{
                if (lesson.LessonType === lesson_type_id || lesson_type_id === 0) {
                    table.showColumn("Lesson"+lesson.ID);
                } else {
                    table.hideColumn("Lesson"+lesson.ID);
                }
            })
        }

        listen($selector) {
            $selector.change(() => {
                let lesson_type_id = parseInt($selector.val());

                let lessonTypeFilter = JSON.parse(localStorage.lessonFilter || "{}");
                lessonTypeFilter[discipline.ID] = lesson_type_id;
                localStorage.lessonFilter = JSON.stringify(lessonTypeFilter);

                this.filterLessons(lesson_type_id);
            });
        }
    }

    lessonFilter = new LessonFilter(table, $('#lessonSelector'));

    //trigger download of data.xlsx file
    $("#download-xlsx").click(function () {
        table.download("xlsx", "data.xlsx", {sheetName: "My Data"});
    });

    //trigger download of data.pdf file
    $("#download-pdf").click(function () {
        //table.download("pdf", "data.pdf", {
        //autoTable: function(doc) {
        // need to use https://github.com/sphilee/jsPDF-CustomFonts-support to generate base64-encoded string
//    doc.addFileToVFS("fontawesome-webfont.ttf", "AAEAAAAOAIAAAwBgRkZUTW..."); // your font in binary format as second parameter
        //doc.addFont("fontawesome-webfont.ttf", "FontAwesome", "normal");
        //doc.setFont("FontAwesome");
        //return {
//      styles: {
        //font: "FontAwesome",
        //fontStyle: "normal"
        //}
        //};
        //}
//});
        table.download("pdf", "data.pdf", {
            orientation: "portrait", //set page orientation to portrait
            title: "Example Report", //add title to report
        });
    });


//TODO: use history callbacks to track changes when using undo/redo
//http://tabulator.info/docs/4.2/callbacks#history

//undo button
    $("#history-undo").on("click", function () {
        table.undo();
    });

//redo button
    $("#history-redo").on("click", function () {
        table.redo();
    });

    $("#new-lesson").on("click", function () {

    });

});
